package at.fhooe.mc.magicwand.client.poi;

import org.kxml.*;
import org.kxml.parser.*;

import at.fhooe.mc.magicwand.client.exceptions.ParsingException;

import java.io.*;
import java.util.Vector;

/**
 * @class XMLParser
 * @author Maria Neumayer
 * @brief parses the XML data into a vector
 */
public class XMLParser {

    /**
     * Current read POI
     */
    private POI m_poi;
    /**
     * Vector to save POI
     */
    private Vector m_poiVector;

    /**
     * @fn public void parseXML(String _xmlString) throws ParsingException
     * @param _xmlString string with xml data
     * @throws ParsingException if Parsing was not successful
     */
    public void parseXML(String _xmlString) throws ParsingException {
        try {

            // String input stream
            byte[] stringBytes = _xmlString.getBytes();
            ByteArrayInputStream bfs = new ByteArrayInputStream(stringBytes);
            InputStreamReader xmlReader = new InputStreamReader(bfs);
            XmlParser parser = new XmlParser(xmlReader);
            ParseEvent pe = null;
            // POI Vector
            m_poiVector = new Vector();

            //parsing
            parser.skip();
            parser.read(Xml.START_TAG, null, "magic-wand");
            boolean trucking = true;

            //go through data until magic-wand end-tags
            while (trucking) {
                pe = parser.read();
                if (pe.getType() == Xml.START_TAG) {
                    String name = pe.getName();
                    if (name.equals("poi")) {
                        m_poi = new POI();

                        //go through a POI
                        while ((pe.getType() != Xml.END_TAG) || (pe.getName().equals(name) == false)) {
                            pe = parser.read();

                            //name
                            if (pe.getType() == Xml.START_TAG && pe.getName().equals("name")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {
                                    m_poi.setM_name(pe.getText());
                                }

                            //description
                            } else if (pe.getType() == Xml.START_TAG && pe.getName().equals("description")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {
                                    m_poi.setM_description(pe.getText());
                                }
                            //link
                            } else if (pe.getType() == Xml.START_TAG && pe.getName().equals("link")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {
                                    m_poi.setM_link(pe.getText());
                                }
                            //category
                            } else if (pe.getType() == Xml.START_TAG && pe.getName().equals("category")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {
                                    m_poi.setM_category(pe.getText());
                                }
                            //priority
                            } else if (pe.getType() == Xml.START_TAG && pe.getName().equals("priority")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {
                                    m_poi.setM_priority(Integer.parseInt(pe.getText()));
                                }
                            //distance
                            } else if (pe.getType() == Xml.START_TAG && pe.getName().equals("distance")) {
                                pe = parser.read();
                                if (pe.getType() != Xml.END_TAG) {

                                    m_poi.setM_distance(Double.parseDouble(pe.getText()));
                                }
                            }
                        }
                        //add POI to vector
                        if (m_poi.getM_name() != null && m_poi.getM_distance() != 0 && m_poi.getM_priority() != 0) {
                            m_poiVector.addElement(m_poi);
                        }
                    } else {
                        while ((pe.getType() != Xml.END_TAG) || (pe.getName().equals(name) == false)) {
                            pe = parser.read();
                        }
                    }
                }
                if (pe.getType() == Xml.END_TAG && pe.getName().equals("magic-wand")) {
                    trucking = false;
                }
            }

        } catch (IOException _e) {
            throw new ParsingException("Error while processing data.");
        }
    }

    /**
     * @fn public Vector getM_poiVector()
     * @return vector with POI
     */
    public Vector getM_poiVector() {
        return m_poiVector;
    }

    /**
     * @fn public void setM_poiVector(Vector vector)
     * @param vector POI vector
     */
    public void setM_poiVector(Vector vector) {
        m_poiVector = vector;
    }
}
